"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guardParamType = void 0;
var config_1 = require("../../config");
var type_guard_1 = require("../../type-guard");
/**
 * Verify the TypeScript param type is matching the NativeType from `ParamType`
 */
function guardParamType(target, propertyKey, parameterIndex, nativeType, pointerTypeList) {
    var _a;
    var designParamTypeList = Reflect.getMetadata('design:paramtypes', target, propertyKey);
    var designParamType = designParamTypeList[parameterIndex];
    config_1.log.verbose('Sidecar', 'guardParamType(%s, %s, %s) %s.%s(args[%s]) designType/nativeType/pointerTypes: %s/%s/%s', target.constructor.name, propertyKey, parameterIndex, target.constructor.name, propertyKey, parameterIndex, (_a = designParamType === null || designParamType === void 0 ? void 0 : designParamType.name) !== null && _a !== void 0 ? _a : 'void', nativeType, pointerTypeList.join(','));
    try {
        (0, type_guard_1.guardNativeType)(nativeType)(designParamType);
        if (nativeType === 'pointer') {
            (0, type_guard_1.guardPointerType)(pointerTypeList)(designParamType);
        }
    }
    catch (e) {
        config_1.log.error('Sidecar', [
            "The \"" + target.constructor.name + "." + String(propertyKey) + "(args[" + parameterIndex + "])",
            "decorated by \"@ParamType(" + nativeType + ", ...)\"",
            "does match the design type \"" + (designParamType === null || designParamType === void 0 ? void 0 : designParamType.name) + "\"",
        ].join('\n'));
        throw e;
    }
}
exports.guardParamType = guardParamType;
//# sourceMappingURL=guard-param-type.js.map