"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSidecarMetadata = void 0;
var config_1 = require("../../config");
var function_target_1 = require("../../function-target");
var metadata_call_1 = require("../call/metadata-call");
var hook_1 = require("../hook/hook");
var metadata_param_type_1 = require("../param-type/metadata-param-type");
var metadata_ret_type_1 = require("../ret-type/metadata-ret-type");
var target_1 = require("./target");
function buildSidecarMetadata(klass, options) {
    var e_1, _a, _b;
    if (options === void 0) { options = {}; }
    config_1.log.verbose('Sidecar', 'buildSidecarMetadata(%s%s)', klass.name, options
        ? ", \"" + JSON.stringify(options).substr(0, 20) + "...\""
        : '');
    var interceptorList = [];
    var nativeFunctionList = [];
    var propertyList = Object.getOwnPropertyNames(klass.prototype);
    try {
        for (var propertyList_1 = __values(propertyList), propertyList_1_1 = propertyList_1.next(); !propertyList_1_1.done; propertyList_1_1 = propertyList_1.next()) {
            var property = propertyList_1_1.value;
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() building "%s.%s"...', klass.name, property);
            /**
             * 1. Get the metadata of `call` and `hook`
             */
            var callMetadata = (0, metadata_call_1.getMetadataCall)(klass.prototype, property);
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() callMetadata of %s: %s', property, JSON.stringify(callMetadata));
            var hookMetadata = (0, hook_1.getMetadataHook)(klass.prototype, property);
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() hookMetadata of %s: %s', property, JSON.stringify(hookMetadata));
            /**
             * 2. Make sure the target exists: either `call` or `hook`
             */
            var target = callMetadata || hookMetadata;
            if (!target) {
                config_1.log.silly('Sidecar', 'buildSidecarMetadata() no callMetadata nor hookMetadata of %s: skip this loop', property);
                continue;
            }
            var targetObj = (0, function_target_1.normalizeFunctionTarget)(target);
            /**
             * 3. Get parameter and return types
             */
            var paramTypeMetadata = (0, metadata_param_type_1.getMetadataParamType)(klass.prototype, property);
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() paramTypeMetadata of %s: %s', property, JSON.stringify(paramTypeMetadata));
            var retTypeMetadata = (0, metadata_ret_type_1.getMetadataRetType)(klass.prototype, property);
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() retTypeMetadata of %s: %s', property, JSON.stringify(retTypeMetadata));
            /**
             * 4. Build the function descript and save it
             */
            var functionDescription = (_b = {},
                _b[targetObj.type] = {
                    name: property,
                    paramTypeList: paramTypeMetadata,
                    retType: retTypeMetadata,
                    target: targetObj,
                },
                _b);
            config_1.log.silly('Sidecar', 'buildSidecarMetadata() functionDescription of %s: %s', property, JSON.stringify(functionDescription));
            if (callMetadata) {
                nativeFunctionList.push(functionDescription);
            }
            if (hookMetadata) {
                interceptorList.push(functionDescription);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (propertyList_1_1 && !propertyList_1_1.done && (_a = propertyList_1.return)) _a.call(propertyList_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    var initAgentScript = options.initAgentScript, sidecarTarget = options.sidecarTarget;
    var normalizedTarget = (0, target_1.normalizeSidecarTarget)(sidecarTarget);
    return {
        initAgentScript: initAgentScript,
        interceptorList: interceptorList,
        nativeFunctionList: nativeFunctionList,
        sidecarTarget: normalizedTarget,
    };
}
exports.buildSidecarMetadata = buildSidecarMetadata;
//# sourceMappingURL=build-sidecar-metadata.js.map