"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.guardMetadataSidecar = void 0;
var config_1 = require("../../config");
/**
 * Verify the Sidecar Metadata is satisfy as a whole
 */
function guardMetadataSidecar(meta) {
    var e_1, _a;
    config_1.log.verbose('Sidecar', 'guardMetadataSidecar(meta)');
    try {
        for (var _b = __values(meta.nativeFunctionList), _c = _b.next(); !_c.done; _c = _b.next()) {
            var desc = _c.value;
            /**
             * Huan(202108): Check for `AgentTarget`: the agentTarget
             *  will be mapped to a JavaScript function
             *  inside the `initAgentScript` source code.
             *
             * Which means that it should not be annoated
             *  by either `@ParamType` or `RetType`
             */
            if (desc.agent) {
                if (desc.agent.paramTypeList.length > 0) {
                    throw new Error([
                        "The sidecar method \"" + desc.agent.name + "\" is decorated as 'AgentType'",
                        'which means that it will be mapped to a JavaScript function',
                        'in `initAgentScript` source code,',
                        'So decorated it with `@ParamType() is not allowed`,',
                        'Remove `@ParamType` and try again.',
                    ].join('\n'));
                }
                if (desc.agent.retType) {
                    throw new Error([
                        "The sidecar method \"" + desc.agent.name + "\" is decorated as 'AgentType'",
                        'which means that it will be mapped to a JavaScript function',
                        'in `initAgentScript` source code,',
                        'So decorated it with `@RetType() is not allowed`,',
                        'Remove `@RetType` and try again.',
                    ].join('\n'));
                }
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
}
exports.guardMetadataSidecar = guardMetadataSidecar;
//# sourceMappingURL=guard-metadata-sidecar.js.map