"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sidecar = void 0;
var config_1 = require("../../config");
var sidecar_body_1 = require("../../sidecar-body/sidecar-body");
var build_sidecar_metadata_1 = require("./build-sidecar-metadata");
var guard_metadata_sidecar_1 = require("./guard-metadata-sidecar");
var metadata_sidecar_1 = require("./metadata-sidecar");
function Sidecar(sidecarTarget, initAgentScript) {
    config_1.log.verbose('Sidecar', '@Sidecar(%s%s)', Array.isArray(sidecarTarget)
        ? JSON.stringify(sidecarTarget)
        : sidecarTarget, initAgentScript
        ? ", \"" + initAgentScript.substr(0, 20) + "...\""
        : '');
    return classDecorator;
    /**
     * See: https://www.typescriptlang.org/docs/handbook/decorators.html#class-decorators
     */
    function classDecorator(Klass) {
        config_1.log.verbose('Sidecar', '@Sidecar(%s%s) classDecorator(%s)', Array.isArray(sidecarTarget)
            ? JSON.stringify(sidecarTarget)
            : (sidecarTarget || ''), "\"" + (initAgentScript === null || initAgentScript === void 0 ? void 0 : initAgentScript.substr(0, 20)) + "...\"" || '', Klass.name);
        // https://stackoverflow.com/a/14486171/1123955
        if (!(Klass.prototype instanceof sidecar_body_1.SidecarBody)) {
            throw new Error('Sidecar: the class decorated by @Sidecar must extends from `SidecarBody`');
        }
        var meta = (0, build_sidecar_metadata_1.buildSidecarMetadata)(Klass, {
            initAgentScript: initAgentScript,
            sidecarTarget: sidecarTarget,
        });
        /**
         * Validate metadata for sidecar
         */
        (0, guard_metadata_sidecar_1.guardMetadataSidecar)(meta);
        /**
         * Save metadata
         */
        (0, metadata_sidecar_1.updateMetadataSidecar)(Klass, meta);
    }
}
exports.Sidecar = Sidecar;
//# sourceMappingURL=sidecar.js.map