"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.guardPointerType = exports.guardNativeType = void 0;
var undefinedNativeTypes = [
    'pointer',
    'void',
];
var undefinedPointerTypes = [];
var bufferNativeTypes = ['pointer'];
var bufferPointerTypes = ['ByteArray'];
var stringNativeTypes = ['pointer'];
var stringPointerTypes = [
    'AnsiString',
    'CString',
    'Utf16String',
    'Utf8String',
];
var numberNativeTypes = [
    'pointer',
    'int',
    'uint',
    'long',
    'ulong',
    'char',
    'uchar',
    'size_t',
    'ssize_t',
    'float',
    'double',
    'int8',
    'uint8',
    'int16',
    'uint16',
    'int32',
    'uint32',
    'int64',
    'uint64',
];
var numberPointerTypes = [
    'S8',
    'U8',
    'S16',
    'U16',
    'S32',
    'U32',
    'Short',
    'UShort',
    'Int',
    'UInt',
    'Float',
    'Double',
    'S64',
    'U64',
    'Long',
    'ULong',
];
var booleanNativeTypes = [
    'pointer',
    'bool',
];
var booleanPointerTypes = [];
/**
 * The main table for storing compatible native & pointer types for design types.
 */
var designTypesCompatibleTable = new Map()
    /**
     * Huan(202107): TypeScript can not get the ReturnType<Promise<???>> today (v4.4)
     */
    .set(Promise, [
    [],
    [],
])
    .set(undefined, [
    undefinedNativeTypes,
    undefinedPointerTypes,
])
    .set(Buffer, [
    bufferNativeTypes,
    bufferPointerTypes,
])
    .set(String, [
    stringNativeTypes,
    stringPointerTypes,
])
    .set(Number, [
    numberNativeTypes,
    numberPointerTypes,
])
    .set(Boolean, [
    booleanNativeTypes,
    booleanPointerTypes,
]);
var guardNativeType = function (nativeType) { return function (designType) {
    if (!designTypesCompatibleTable.has(designType)) {
        throw new Error("Unsupported designType: " + (typeof designType) + " " + (designType && designType.name) + " " + designType);
    }
    var typeList = designTypesCompatibleTable.get(designType);
    if (!typeList) {
        throw new Error('nativeType can not found from designToNativeTypeTable[' + designType + ']');
    }
    var _a = __read(typeList, 2), nativeTypeList = _a[0], _ = _a[1];
    // [] means allow all
    if (nativeTypeList.length > 0 && !nativeTypeList.includes(nativeType)) {
        throw new Error("NativeType(" + nativeType + ") does match the design type \"" + (designType === null || designType === void 0 ? void 0 : designType.name) + "\"");
    }
}; };
exports.guardNativeType = guardNativeType;
var guardPointerType = function (pointerTypeList) { return function (designType) {
    var pointerType;
    if (pointerTypeList.length > 0) {
        pointerType = pointerTypeList[pointerTypeList.length - 1];
    }
    else {
        /**
         * Huan(202107): NativePointer allow raw pointer (`null` as well)
         */
        pointerType = 'Pointer';
    }
    if (!designTypesCompatibleTable.has(designType)) {
        throw new Error("Unsupported designType: " + (typeof designType) + " " + (designType && designType.name) + " " + designType);
    }
    var typeList = designTypesCompatibleTable.get(designType);
    if (!typeList) {
        throw new Error('pointerType can not found from designToPointerTypeTable[' + designType + ']');
    }
    // console.log(typeList)
    var _a = __read(typeList, 2), _ = _a[0], compatiblePointerTypeList = _a[1];
    /**
     * Huan(202106): why `typeList.length > 0`?
     *  nativeTypeList will be empty for the designType `Promise`
     *  because the TypeScript metadata do not support to get the value inside the `Promise<value>`
     *  so we will not be able to check them.
     */
    // [] means allow all
    if (compatiblePointerTypeList.length > 0 && !compatiblePointerTypeList.includes(pointerType)) {
        throw new Error("PointerType(" + pointerType + ") does match the design type \"" + (designType === null || designType === void 0 ? void 0 : designType.name) + "\"");
    }
}; };
exports.guardPointerType = guardPointerType;
//# sourceMappingURL=type-guard.js.map